Package: shades
Version: 1.5.0
Date: 2026-07-01
Title: Simple Colour Manipulation
Authors@R: c(person("Jon", "Clayden", role=c("cre","aut"),
    email="code@clayden.org", comment=c(ORCID="0000-0002-6608-0619")))
Description: Functions for easily manipulating colours, creating colour scales and calculating colour distances.
Suggests: tinytest, covr
Enhances: colorspace, ggplot2, tinyplot
Encoding: UTF-8
License: BSD_3_clause + file LICENCE
URL: https://github.com/jonclayden/shades
BugReports: https://github.com/jonclayden/shades/issues
Config/roxygen2/version: 8.0.0
Config/roxygen2/markdown: TRUE
NeedsCompilation: no
Packaged: 2026-07-01 21:46:15 UTC; jon
Author: Jon Clayden [cre, aut] (ORCID: <https://orcid.org/0000-0002-6608-0619>)
Maintainer: Jon Clayden <code@clayden.org>
Repository: CRAN
Date/Publication: 2026-07-02 21:50:07 UTC
