# Generated by roxygen2: do not edit by hand

S3method("!=",shade)
S3method("==",shade)
S3method("[",shade)
S3method("[<-",shade)
S3method(all.equal,shade)
S3method(c,shade)
S3method(coords,default)
S3method(coords,shade)
S3method(print,shade)
S3method(rep,shade)
S3method(rev,shade)
S3method(shade,character)
S3method(shade,color)
S3method(shade,default)
S3method(shade,matrix)
S3method(shade,shade)
S3method(space,default)
S3method(space,shade)
export("%.)%")
export("%_/%")
export(addmix)
export(brightness)
export(chroma)
export(complement)
export(coords)
export(delta)
export(dichromat)
export(distance)
export(gradient)
export(hue)
export(lightness)
export(opacity)
export(recycle)
export(saturation)
export(scalefac)
export(shade)
export(space)
export(submix)
export(swatch)
export(warp)
import(grDevices)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,rect)
